/*
 * These are the unit tests for the integration. They should NOT test the network nor any
 * remote third-party functionality - only that the local code acts and runs as expected.
 */

'use strict'

var Analytics = require('@segment/analytics.js-core').constructor
var Integration = require('@segment/analytics.js-integration')
var Sandbox = require('@segment/clear-env')
var Tester = require('@segment/analytics.js-integration-tester')
var HotJar = require('../lib/')

describe('HotJar Unit', function () {
  var analytics
  var hotJar
  var customOptions
  var options = {
    hjid: 485778,
    hjPlaceholderPolyfill: true
  }

  beforeEach(function () {
    analytics = new Analytics()
    hotJar = new HotJar(options)
    analytics.use(HotJar)
    analytics.use(Tester)
    analytics.add(hotJar)

    customOptions = {
      hjid: 485778,
      hjPlaceholderPolyfill: false
    }
  })

  afterEach(function () {
    analytics.restore()
    analytics.reset()
    hotJar.reset()
    Sandbox()
  })

  describe('Constructing the integration', function () {
    it('should have the right settings', function () {
      analytics.compare(HotJar, Integration('HotJar')
      .option('hjid', null)
      .option('hjPlaceholderPolyfill', true))
    })

    it('should have the right settings with custom options', function () {
      hotJar = new HotJar(customOptions) // Intentionally creating a new HotJar integration here to not simply compare the object to itself.

      analytics.deepEqual(hotJar.options, customOptions)
    })
  })

  describe('#initialize', function () {
    it('should call this.ready() with valid options', function () {
      analytics.stub(hotJar, 'load')
      analytics.stub(hotJar, 'ready')
      hotJar.initialize()
      analytics.called(hotJar.load)
      analytics.called(hotJar.ready)
    })

    it('should prepare the needed globals', function () {
      hotJar.initialize()
      analytics.deepEqual(window._hjSettings, {hjid: options.hjid, hjsv: 5, hjPlaceholderPolyfill: true})
      analytics.assert(typeof window.hj === 'function')
    })

    it('should reject an invalid HJID', function () {
      customOptions.hjid = NaN
      testInvalidInitialize(customOptions)
    })

    it('should reject an invalid hjPlaceholderPolyfill boolean', function () {
      customOptions.hjPlaceholderPolyfill = 1
      testInvalidInitialize(customOptions)
    })

    function testInvalidInitialize (invalidOptions) {
      hotJar.options = invalidOptions
      analytics.stub(hotJar, 'ready')
      hotJar.initialize()
      analytics.didNotCall(hotJar.ready)
    }
  })
})
